import click
import docker

from wheezy.template.engine import Engine
from wheezy.template.ext.core import CoreExtension
from wheezy.template.ext.code import CodeExtension
from wheezy.template.loader import DictLoader

from . import templates

import logging

LOG = logging.getLogger(__name__)


LOG_LEVELS = {
    "info": logging.INFO,
    "warn": logging.WARN,
    "debug": logging.DEBUG,
    "error": logging.ERROR,
    "critical": logging.CRITICAL,
}


class Context(dict):
    def __init__(self, *args, **kwargs):
        self.__dict__ = self
        super(Context, self).__init__(*args, **kwargs)


class FermConfig(object):
    def __init__(self, path):
        self.path = path

        template_dct = {
            'docker': templates.docker,
        }
        engine = Engine(
            loader=DictLoader(template_dct),
            extensions=[
                CoreExtension(),
                CodeExtension()
            ]
        )
        self.templates = {
            name: engine.get_template(name) for name in template_dct
        }

    def get_config(self, config):
        return self.templates['docker'].render(config)


@click.group()
@click.option(
    "--log-level",
    type=click.Choice([k for k, v in sorted(LOG_LEVELS.iteritems(), key=lambda x: x[1])]),
    default="info",
    help="Logging level.")
@click.pass_context
def run(ctx, log_level):
    logging.basicConfig(level=LOG_LEVELS[log_level])

    ctx.obj = Context()


@run.group("docker")
@click.option(
    "api", "--docker", "-d",
    type=click.Path(),
    default="unix://var/run/docker.sock",
    help="The docker api socket."
)
@click.option(
    "--cidr", "-c", default="172.18.0.0/16",
    help="Docker CIDR."
)
@click.option(
    "--interface", "-i", default="docker0",
    help="Docker interface."
)
@click.pass_context
def docker_grp(ctx, api, cidr, interface):
    ctx.obj.client = docker.Client(base_url=api)
    ctx.obj.cidr = cidr
    ctx.obj.interface = interface


@docker_grp.command(name="config")
@click.pass_context
def docker_config(ctx):
    ferm = FermConfig(None)

    # get all containers
    containers = ctx.obj.client.containers()

    ctx.obj.containers = [
        ctx.obj.client.inspect_container(container['Id'])
        for container in containers
    ]
    click.echo(ferm.get_config(ctx.obj))



"""
Some wheezy templates.
"""

docker = """\
@require(cidr, interface, containers)
domain ip {
    table nat {
        chain DOCKER;
        chain PREROUTING {
            policy ACCEPT;
            mod addrtype dst-type LOCAL jump DOCKER;
        }
        chain OUTPUT {
            policy ACCEPT;
            daddr ! 127.0.0.0/8 mod addrtype dst-type LOCAL jump DOCKER;
        }
        chain POSTROUTING {
            policy ACCEPT;
            saddr @cidr outerface ! @interface MASQUERADE;
        }
    }
    table filter {
        chain DOCKER;
        chain DOCKER-ISOLATION-STAGE-1;
        chain DOCKER-ISOLATION-STAGE-2;
        chain DOCKER-USER;
        chain FORWARD {
	    jump DOCKER-USER;
            jump DOCKER-ISOLATION-STAGE-1;
            outerface @interface {
                mod conntrack ctstate (RELATED ESTABLISHED) ACCEPT;
		jump DOCKER;
            }
            interface @interface {
                outerface ! @interface ACCEPT;
                outerface @interface ACCEPT;
            }
        }

	chain DOCKER-ISOLATION-STAGE-1 {
		interface docker0 outerface ! docker0 jump DOCKER-ISOLATION-STAGE-2;
	           jump RETURN;
        	}
	chain DOCKER-ISOLATION-STAGE-2 {
	        outerface docker0 DROP;
	           jump RETURN;	
	        }
	chain DOCKER-USER {
	           jump RETURN;
	}

    }

    # container setup
    @for container in containers:
    # @container['Id']
    @(
        networks = container['NetworkSettings']['Networks'].keys()
        first_network = list(networks)[0]
        # TODO: Loop over each network instad just using the first network!
        ip_address = container['NetworkSettings']['Networks'][first_network]['IPAddress']
        port_bindings = container['HostConfig']['PortBindings']
        # group into proto:port:(host:port)
        bindings = {}
        if port_bindings != None:
            for port_proto, binds in port_bindings.iteritems():
                port, proto = port_proto.split("/")
                if proto not in bindings:
                    bindings[proto] = {}
                if port not in bindings[proto]:
                    bindings[proto][port] = []
                for bind in binds:
                    bindings[proto][port].append((bind['HostIp'], bind['HostPort']))
    )
    table nat {
        chain POSTROUTING {
            # container setup
            @for proto, ports in bindings.iteritems():
            @for port, binds in ports.iteritems():
            saddr @ip_address/32 daddr @ip_address/32 protocol @proto {
                dport @port MASQUERADE;
            }
            @end
            @end
        }
        chain DOCKER {
            @for proto, ports in bindings.iteritems():
            @for port, binds in ports.iteritems():
            @for host_ip, host_port in binds:
            @{ host_ip and "daddr %s/32 " % host_ip or '' }interface ! @interface protocol tcp dport @host_port DNAT to @ip_address:@port;
            @end
            @end
            @end
        }
    }
    table filter {
        chain DOCKER {
            @for proto, ports in bindings.iteritems():
            daddr @ip_address/32 interface ! @interface outerface @interface protocol @proto {
            @for port, binds in ports.iteritems():
                dport @port ACCEPT;
            @end
            }
            @end
        }
    }
    @end
}
"""




from setuptools import setup, find_packages

__version__ = "0.0.2"


setup(
    version=__version__,
    name="ferment_ng",
    description="A tool to provide the current docker config in ferm format",
    author="Olga Ukhina",
    author_email="olga.uhina@gmail.com",
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
    ],
    keywords="ferm docker iptables",
    license="Apache Version 2",
    package_dir={'': 'src'},
    packages=find_packages(
        "src",
        exclude=[]
    ),
    namespace_packages=["ferment_ng"],
    include_package_data=True,
    zip_safe=False,
    entry_points="""\
    [console_scripts]
    ferment_ng=ferment_ng.scripts:run
    """,
    install_requires=[
        "docker-py",
        "click",
        "wheezy.template",
    ],
)
